#include <libnet.h>
#include <sys/poll.h>
#include "get_ipid.h"

int *get_ipid_timeout;
int *get_ipid_read_socket;

int get_ipid(unsigned short *ipid, unsigned int s_addr, unsigned int d_addr, int timeout, int d_port, int s_tcpflags, int verbose)
{
  unsigned char *buf;
  unsigned int s_ipid, s_ttl, s_port, s_seq, s_ack, s_window;
  struct iphdr *iphdr;
  struct tcphdr *tcphdr;
  int read_socket, tcp_socket;
  int ret = 0;
  struct timeval tv, tv2;

  get_ipid_timeout = &timeout;
  get_ipid_read_socket = &read_socket;

  s_ipid = rand() & 0xffff;
  s_ttl = 100;
  s_port = rand() & 0xffff;
  s_seq = rand();
  s_ack = rand();
  s_window = 16384;

  if (-1 == libnet_init_packet(IP_MAXPACKET, &buf)) {
    perror("No memory for packet header");
    exit(1);
  }

  if ((tcp_socket = libnet_open_raw_sock(IPPROTO_RAW)) == -1) {
    perror("No socket");
    exit(1);
  }

  libnet_build_ip(LIBNET_TCP_H,                      /* Size of the payload */
		  IPTOS_LOWDELAY | IPTOS_THROUGHPUT, /* IP tos */
		  s_ipid,                            /* IP ID */
		  0,                                 /* Frag stuff */
		  s_ttl,                             /* TTL */
		  IPPROTO_TCP,                       /* Transport protocol */
		  s_addr,                           /* Source IP */
		  d_addr,                           /* Destination IP */
		  NULL,                              /* Pointer to payload (none) */
		  0,                                 /* payload size */
		  buf);                              /* Packet header memory */

  libnet_build_tcp(s_port,                        /* Source TCP port */
		   d_port,                           /* Destination TCP port */
		   s_seq,                            /* Sequence number */
		   s_ack,                            /* Acknowledgement number */
		   s_tcpflags,                       /* Control flags */
		   s_window,                         /* Window size */
		   0,                                /* Urgent pointer */
		   NULL,                             /* Pointer to payload (none) */
		   0,                                /* payload size */
		   buf + LIBNET_IP_H);               /* Packet header memory */
  
  libnet_do_checksum(buf, IPPROTO_TCP, LIBNET_TCP_H);
  
  if (0 > (read_socket = socket(AF_INET, SOCK_RAW, IPPROTO_TCP))) {
    perror("socket(raw)");
    exit(1);
  }

  if ((libnet_write_ip(tcp_socket, buf, LIBNET_TCP_H + LIBNET_IP_H)) < LIBNET_TCP_H + LIBNET_IP_H) {
    fprintf(stderr, "libnet_write_ip\n");
  }

  gettimeofday(&tv, NULL);

  for (;;) {
    struct pollfd p;

    p.fd = read_socket;
    p.events = POLLIN;

    if (!poll(&p, 1, 500)) {
      continue;
    }
    gettimeofday(&tv2, NULL);
    if (tv2.tv_sec > (tv.tv_sec + timeout)) {
      if (verbose) {
	printf("timeout\n");
      }
      ret = -1;
      break; /* exit inf for loop */
    }
    
    //      printf("done\n");
    //    printf("%d %d\n", (int)tv.tv_sec, (int)tv.tv_usec);
    memset(buf, 0, LIBNET_TCP_H + LIBNET_IP_H);
    if (-1 == read(read_socket, buf, 65536)) {
      /* prolly due to timeout  */
      ret = -1;
      break;
    }

    iphdr = (struct iphdr*)buf;
    tcphdr = (struct tcphdr*)&buf[sizeof(struct iphdr)];
    
    /*
     * check if correct packet
     */
    if ((iphdr->saddr == d_addr) && (tcphdr->th_dport == htons(s_port))) {
      *ipid = (unsigned short)htons(((short*)(buf))[2]); /* return the ID */
      if (verbose >= 2) {
	printf("Recv: %s:%4x:%8x  id: 0x%.4x\n", libnet_host_lookup(iphdr->saddr, 1),
	       tcphdr->th_dport, htonl(tcphdr->th_ack),
	       *ipid);
      }
      break;
    }
  }
  libnet_destroy_packet(&buf);
  close(read_socket);
  libnet_close_raw_sock(tcp_socket);
  return ret;
}
